/*------------------------------------------------------------------------
    File        : GenericEntity
    Purpose     : 
    Syntax      : 
    Description : 
    Author(s)   : egarcia
    Created     : Mon Aug 31 11:21:00 EST 2015
    Notes       : 
  ----------------------------------------------------------------------*/

USING Progress.Json.ObjectModel.*.
USING Progress.Lang.*.


BLOCK-LEVEL ON ERROR UNDO, THROW.

CLASS GenericEntity: 

    DEFINE VARIABLE GenericServiceName    AS CHARACTER NO-UNDO INITIAL "GenericService".
    DEFINE VARIABLE GenericServiceAddress AS CHARACTER NO-UNDO INITIAL "/data".
	
    DEFINE VARIABLE tableName             AS CHARACTER NO-UNDO.
	
    CONSTRUCTOR PUBLIC GenericEntity(INPUT tableName AS CHARACTER):
        THIS-OBJECT:tableName = tableName.
    END.

    METHOD PUBLIC LONGCHAR Catalog():
        DEFINE VARIABLE jsonText AS LONGCHAR	NO-UNDO.
		
        CatalogForTable(THIS-OBJECT:tableName, OUTPUT jsonText).
        RETURN jsonText.
    END.
	
    METHOD PUBLIC LONGCHAR ReadJSON(INPUT filter AS CHARACTER):
        DEFINE VARIABLE hDataSet   AS HANDLE     NO-UNDO.        
        DEFINE VARIABLE jsonObject AS JsonObject NO-UNDO.    

        ReadTable(THIS-OBJECT:tableName, filter, OUTPUT DATASET-HANDLE hDataSet).
        
        jsonObject = NEW JsonObject().
        jsonObject:READ(hDataSet).
        
        RETURN jsonObject:GetJsonText().
    END.

    METHOD PRIVATE VOID CatalogForTable(INPUT tableName AS CHARACTER, OUTPUT jsonText AS LONGCHAR):    
        DEFINE VARIABLE jsonObject  AS JsonObject        NO-UNDO.
        DEFINE VARIABLE jsonObject1 AS JsonObject        NO-UNDO.    
        DEFINE VARIABLE jsonObject2 AS JsonObject        NO-UNDO.

        DEFINE VARIABLE services    AS JsonArray         NO-UNDO.        
        DEFINE VARIABLE resources   AS JsonArray         NO-UNDO.
        DEFINE VARIABLE operations  AS JsonArray         NO-UNDO.
        DEFINE VARIABLE params      AS JsonArray         NO-UNDO.            

        DEFINE VARIABLE service     AS JsonObject        NO-UNDO.    
        DEFINE VARIABLE resource    AS JsonObject        NO-UNDO.
        DEFINE VARIABLE schema1     AS JsonObject        NO-UNDO.
        DEFINE VARIABLE properties1 AS JsonObject        NO-UNDO.
        DEFINE VARIABLE dataSet1    AS JsonObject        NO-UNDO.
        DEFINE VARIABLE tempTable1  AS JsonObject        NO-UNDO.    
        DEFINE VARIABLE properties2 AS JsonObject        NO-UNDO.
        DEFINE VARIABLE items       AS JsonObject        NO-UNDO.
        DEFINE VARIABLE properties3 AS JsonObject        NO-UNDO.        
		
        DEFINE VARIABLE parser      AS ObjectModelParser NO-UNDO.
        DEFINE VARIABLE construct1  AS JsonConstruct     NO-UNDO.
		
        DEFINE VARIABLE hBuffer     AS HANDLE            NO-UNDO.
        DEFINE VARIABLE hq          AS HANDLE            NO-UNDO.                

        DEFINE VARIABLE baseName    AS CHARACTER         NO-UNDO.
			
        /* tableName = ENTRY(1, tableName, "."). */    
			
        baseName = CAPS(SUBSTRING(tableName, 1, 1)) + SUBSTRING(tableName, 2).

        /*parser = NEW ObjectModelParser().
        construct1 = parser:Parse('~{"type":"object", "additionalProperties":false, "properties":~{~}~}').
        schema1 = CAST(construct1, JsonObject).
        */

        CREATE BUFFER hBuffer FOR TABLE tableName.
        FindTemplate(hBuffer).

        properties3 = NEW JsonObject().
        jsonObject1 = NEW JsonObject().
        jsonObject1:Add("type", "string").
        properties3:Add("_id", jsonObject1).                                                  
        FOR EACH _file NO-LOCK WHERE _file-name = tableName:
            FOR EACH _field OF _file NO-LOCK BY _order:
                jsonObject1 = NEW JsonObject().            

                IF _field._extent > 0 THEN 
                DO:
                    jsonObject1:Add("type", "array").
                    jsonObject1:Add("maxItems", _field._extent).
                    jsonObject2 = NEW JsonObject().
                    jsonObject2:Add("type", GetJavaScriptType(_field._data-type)).
                    IF _field._data-type = "DATE" THEN 
                    DO:
                        jsonObject2:Add("format", "date").
                    END.
                    IF _field._data-type = "DATETIME" OR _field._data-type = "DATETIME-TZ" THEN 
                    DO:
                        jsonObject2:Add("format", "date-time").
                    END.
                    jsonObject1:Add("items", jsonObject2).
                    jsonObject1:Add("default", hBuffer:BUFFER-FIELD (_field._field-name):BUFFER-VALUE(1)).
                END.
                ELSE 
                DO:
                    jsonObject1:Add("type", GetJavaScriptType(_field._data-type)).
                    jsonObject1:Add("ablType", CAPS(_field._data-type)).
                    jsonObject1:Add("default", hBuffer:BUFFER-FIELD (_field._field-name):BUFFER-VALUE).
                    IF _field._data-type = "DATE" THEN 
                    DO:
                        jsonObject1:Add("format", "date").
                    END.
                    IF _field._data-type = "DATETIME" OR _field._data-type = "DATETIME-TZ" THEN 
                    DO:
                        jsonObject1:Add("format", "date-time").
                    END.
                END.

                jsonObject1:Add("title", _field._label).                        
                properties3:Add(_field._field-name, jsonObject1).                          
            END.          
        END.

        items = NEW JsonObject().
        items:Add("additionalProperties", FALSE).
        items:Add("properties", properties3).        

        tempTable1 = NEW JsonObject().
        tempTable1:Add("type", "array").    
        tempTable1:Add("items", items).        

        properties2 = NEW JsonObject().
        properties2:Add("tt" + baseName, tempTable1).
		
        dataSet1 = NEW JsonObject().
        dataSet1:Add("type", "object").
        dataSet1:Add("additionalProperties", FALSE).
        dataSet1:Add("properties", properties2).    

        properties1 = NEW JsonObject().
        properties1:Add("ds" + baseName, dataSet1).

        schema1 = NEW JsonObject().
        schema1:Add("type", "object").
        schema1:Add("additionalPoperties", FALSE).
        schema1:Add("properties", properties1).    

        operations = NEW JsonArray().
		
        params = NEW JsonArray().
        jsonObject1 = NEW JsonObject().
        /*jsonObject1:Add("path", "?filter=~{filter~}").*/
        jsonObject1:Add("path", "").    
        jsonObject1:Add("type", "read").    
        jsonObject1:Add("verb", "get").
        jsonObject1:Add("params", params).
        operations:Add(jsonObject1).        

        params = NEW JsonArray().
        jsonObject1 = NEW JsonObject().
        jsonObject1:Add("name", "ds" + baseName).
        jsonObject1:Add("type", "REQUEST_BODY").
        params:Add(jsonObject1).   

        jsonObject1 = NEW JsonObject().
        jsonObject1:Add("path", "").
        jsonObject1:Add("type", "create").    
        jsonObject1:Add("verb", "post").
        jsonObject1:Add("params", params).
        operations:Add(jsonObject1).
		
        jsonObject1 = NEW JsonObject().
        jsonObject1:Add("path", "").
        jsonObject1:Add("type", "update").    
        jsonObject1:Add("verb", "put").
        jsonObject1:Add("params", params).
        operations:Add(jsonObject1).            
					
        jsonObject1 = NEW JsonObject().
        jsonObject1:Add("path", "").
        jsonObject1:Add("type", "delete").    
        jsonObject1:Add("verb", "delete").
        jsonObject1:Add("params", params).
        operations:Add(jsonObject1).            
		
        resource = NEW JsonObject().
        resource:Add("name", tableName).
        resource:Add("path", "/" + tableName).
        resource:Add("schema", schema1).
        resource:Add("operations", operations).      
		
        resources = NEW JsonArray().
        resources:Add(resource).    
		
        service = NEW JsonObject().
        service:Add("name", GenericServiceName).
        service:Add("address", GenericServiceAddress).
        service:Add("useRequest", TRUE).
        service:Add("resources", resources).            

        services = NEW JsonArray().
        services:Add(service).

        jsonObject = NEW JsonObject().
        jsonObject:Add("version", "1.0").
        jsonObject:Add("lastModified", ISO-DATE(NOW)).
        jsonObject:Add("services", services).        
		
        jsonText = jsonObject:GetJsonText().        
    END.    
    
    METHOD PRIVATE VOID FindTemplate(INPUT hBuffer AS HANDLE):		
        DEFINE VARIABLE qh     AS HANDLE    NO-UNDO.    
        DEFINE VARIABLE wRecId AS CHARACTER NO-UNDO.
		
        FIND FIRST _file WHERE _file-name = hBuffer:NAME NO-LOCK.
        wRecId = STRING(_file._template).
		
        CREATE QUERY qh.
        qh:SET-BUFFERS(hBuffer).
        qh:QUERY-PREPARE("FOR EACH " + hBuffer:NAME + " WHERE RECID(" + hBuffer:NAME +  ") = " + wRecId).
        qh:QUERY-OPEN().
        qh:GET-FIRST().    
    END.
	
    METHOD PRIVATE CHARACTER GetJavaScriptType(ablType AS CHARACTER):
        CASE ablType:
            WHEN "CHARACTER" 	THEN 
                RETURN "string".
            WHEN "DECIMAL" 		THEN 
                RETURN "NUMBER".        
            WHEN "INTEGER" 		THEN 
                RETURN "integer".
            WHEN "DATE" 		THEN 
                RETURN "string".
            WHEN "DATETIME" 	THEN 
                RETURN "string".
            WHEN "DATETIME-TZ" 	THEN 
                RETURN "string".
        END CASE.
    END.	
	
    METHOD PRIVATE VOID ReadTable(INPUT tableName    AS CHARACTER,
        INPUT filter       AS CHARACTER,
        OUTPUT DATASET-HANDLE hDataSet):    
        DEFINE VARIABLE hSrcBuffer    AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hTargetBuffer AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hTempTable    AS HANDLE    NO-UNDO.    
		
        DEFINE VARIABLE baseName      AS CHARACTER NO-UNDO.    
        baseName = CAPS(SUBSTRING(tableName, 1, 1)) + SUBSTRING(tableName, 2).

        /* Create TEMP-TABLE */
        CREATE TEMP-TABLE hTempTable.
        hTempTable:CREATE-LIKE(tableName).
        hTempTable:TEMP-TABLE-PREPARE("tt" + baseName).
        hTargetBuffer = hTempTable:DEFAULT-BUFFER-HANDLE.
		
        /* Create DataSet */
        CREATE DATASET hDataSet.
        hDataSet:NAME = "ds" + baseName.
        hDataSet:SET-BUFFERS(hTargetBuffer).    

        /* Create buffer for src table */
        CREATE BUFFER hSrcBuffer FOR TABLE tableName.

        PopulateTempTable(hSrcBuffer, hTargetBuffer, filter).
    END METHOD.

    METHOD PRIVATE VOID PopulateTempTable(
        INPUT hSrcBuffer       AS HANDLE,
        INPUT hTargetBuffer    AS HANDLE,
        INPUT filter           AS CHARACTER):
		
        DEFINE VARIABLE qh AS HANDLE NO-UNDO.
		
        CREATE QUERY qh.
        qh:SET-BUFFERS(hSrcBuffer).
        IF filter > "" THEN
            qh:QUERY-PREPARE("FOR EACH " + hSrcBuffer:NAME + " WHERE " + filter).
        ELSE        
            qh:QUERY-PREPARE("FOR EACH " + hSrcBuffer:NAME).        
        qh:QUERY-OPEN().
		
        REPEAT:
            qh:GET-NEXT().
            IF qh:QUERY-OFF-END THEN LEAVE.
				
            hTargetBuffer:BUFFER-CREATE().
            hTargetBuffer:BUFFER-COPY(hSrcBuffer).
        END.
		
        DELETE OBJECT qh.
    END METHOD.	
	
END CLASS.